import { Injectable, Logger } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import User from './users.entity';
import { NotFoundError } from '../utils/errors/notFound.error';
import {
  AlreadyExistsError,
  extractDetailsQueryError,
} from '../utils/errors/alreadyExists.error';
import PostgresErrorCode from '../utils/errors/postgresErrorCode.enum';

@Injectable()
export class UsersService {
  constructor(
    @InjectRepository(User)
    private usersRepository: Repository<User>,
  ) {}
  private static readonly logger: Logger = new Logger(UsersService.name);

  async getByDiscordId(discordId: string) {
    const user = await this.usersRepository.findOneBy({ discordId });
    if (user) {
      return user;
    }
    throw new NotFoundError('User', 'discordId', discordId);
  }

  async getByUserName(username: string) {
    const user = await this.usersRepository.findOneBy({ username });
    if (user) {
      return user;
    }
    throw new NotFoundError('User', 'username', username);
  }

  async create(user: User) {
    try {
      const newUser = this.usersRepository.create(user);
      await this.usersRepository.save(newUser);
      UsersService.logger.log(`New user ${user.username} created`);
      return newUser;
    } catch (error) {
      if (error.code === PostgresErrorCode.UniqueViolation) {
        const details = extractDetailsQueryError(error.detail);
        throw new AlreadyExistsError('User', details.property, details.value);
      }
      throw error;
    }
  }
}
